//
//  KeyboardView.m
//  AnalogSynthesizer
//
//  Created by 成田 拓郎 on 10/05/23.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "KeyboardView.h"


@implementation KeyboardView

@synthesize keyDown;
@synthesize whiteKeyDown1;
@synthesize whiteKeyDown2;
@synthesize whiteKeyDown3;
@synthesize whiteKeyDown4;
@synthesize blackKeyDown;
@synthesize whiteKeyUp1;
@synthesize whiteKeyUp2;
@synthesize whiteKeyUp3;
@synthesize whiteKeyUp4;
@synthesize blackKeyUp;

- (id)initWithFrame:(CGRect)frame {
    if ((self = [super initWithFrame:frame])) {
        // Initialization code
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
	if ((self = [super initWithCoder:aDecoder])) {
		prevTouchPointCount = 0;
		self.keyDown = 0;
		self.whiteKeyDown1 = [UIImage imageNamed:@"whiteKeyDown1.png"];
		self.whiteKeyDown2 = [UIImage imageNamed:@"whiteKeyDown2.png"];
		self.whiteKeyDown3 = [UIImage imageNamed:@"whiteKeyDown3.png"];
		self.whiteKeyDown4 = [UIImage imageNamed:@"whiteKeyDown4.png"];
		self.blackKeyDown = [UIImage imageNamed:@"blackKeyDown.png"];
		self.whiteKeyUp1 = [UIImage imageNamed:@"whiteKeyUp1.png"];
		self.whiteKeyUp2 = [UIImage imageNamed:@"whiteKeyUp2.png"];
		self.whiteKeyUp3 = [UIImage imageNamed:@"whiteKeyUp3.png"];
		self.whiteKeyUp4 = [UIImage imageNamed:@"whiteKeyUp4.png"];
		self.blackKeyUp = [UIImage imageNamed:@"blackKeyUp.png"];
	}
	return self;
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
	static CGFloat x[25] = {
		0,  1,  2,  3,  4,  6,  7,  8,  9,  10, 11, 12,
		14, 15, 16, 17, 18, 20, 21, 22, 23, 24, 25, 26,
		28 };
	static int key_type[25] = {
		2, 4, 3, 4, 1, 2, 4, 3, 4, 3, 4, 1,
		2, 4, 3, 4, 1, 2, 4, 3, 4, 3, 4, 1,
		0 };
	CGFloat key_width = self.bounds.size.width / 30.0f;
	CGFloat height = self.bounds.size.height;
	for (int i=0; i<25; ++i) {
		BOOL down = (keyDown & (1 << i)) != 0;
		CGRect rect = CGRectMake(x[i] * key_width, 0, key_width * 2, height);
		switch (key_type[i]) {
			case 0:
				if (down) [whiteKeyDown1 drawInRect:rect];
				else      [whiteKeyUp1 drawInRect:rect];
				break;
			case 1:
				if (down) [whiteKeyDown2 drawInRect:rect];
				else      [whiteKeyUp2 drawInRect:rect];
				break;
			case 2:
				if (down) [whiteKeyDown3 drawInRect:rect];
				else      [whiteKeyUp3 drawInRect:rect];
				break;
			case 3:
				if (down) [whiteKeyDown4 drawInRect:rect];
				else      [whiteKeyUp4 drawInRect:rect];
				break;
			case 4:
				if (down) [blackKeyDown drawInRect:rect];
				else      [blackKeyUp drawInRect:rect];
				break;
			default:
				break;
		}
	}
}

- (void)updateKeyDown {
	static UInt32 num_black[30] = {
		0,  1,  1,  3,  3,  4,  5,  6,  6,  8,  8,  10, 10, 11,
		12, 13, 13, 15, 15, 16, 17, 18, 18, 20, 20, 22, 22, 23,
		24, 24 };
	static UInt32 num_white[30] = {
		0,  0,  2,  2,  4,  4,  5,  5,  7,  7,  9,  9,  11, 11,
		12, 12, 14, 14, 16, 16, 17, 17, 19, 19, 21, 21, 23, 23,
		24, 24 };
	CGFloat width = self.bounds.size.width;
	CGFloat height = self.bounds.size.height;
	CGFloat key_width = width / 30.0f;
	UInt32 key = 0;
	for (UInt32 i=0; i<prevTouchPointCount; ++i) {
		CGFloat x = prevTouchPoint[i].x;
		CGFloat y = prevTouchPoint[i].y;
		if (x < 0 || x > width || y < 0 || y > height)
			continue;
		int keynum = (int)(x / key_width);
		if (y < 42)
			key = key | (1 << num_black[keynum]);
		else
			key = key | (1 << num_white[keynum]);
	}
	self.keyDown = key;
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	for (UITouch* touch in touches) {
		if (prevTouchPointCount >= 10)
			break;
		prevTouchPoint[prevTouchPointCount] = [touch locationInView:self];
		++prevTouchPointCount;
	}
	[self updateKeyDown];
	[self setNeedsDisplay];

	(void)[target performSelector:action withObject:self];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	BOOL flag[10] = { NO, NO, NO, NO, NO, NO, NO, NO, NO, NO };
	for (UITouch* touch in touches) {
		CGPoint prev_point = [touch previousLocationInView:self];
		UInt32 num = UINT32_MAX;
		CGFloat length = CGFLOAT_MAX;
		for (UInt32 i=0; i<prevTouchPointCount; ++i) {
			if (flag[i])
				continue;
			CGFloat x = prevTouchPoint[i].x - prev_point.x;
			CGFloat y = prevTouchPoint[i].y - prev_point.y;
			CGFloat l = x * x + y * y;
			if (l < length) {
				num = i;
				length = l;
			}
		}
		if (num < 10) {
			flag[num] = YES;
			prevTouchPoint[num] = [touch locationInView:self];
		}
	}
	[self updateKeyDown];
	[self setNeedsDisplay];

	(void)[target performSelector:action withObject:self];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	BOOL flag[10] = { NO, NO, NO, NO, NO, NO, NO, NO, NO, NO };
	for (UITouch* touch in touches) {
		CGPoint prev_point = [touch previousLocationInView:self];
		UInt32 num = UINT32_MAX;
		CGFloat length = CGFLOAT_MAX;
		for (UInt32 i=0; i<prevTouchPointCount; ++i) {
			if (flag[i])
				continue;
			CGFloat x = prevTouchPoint[i].x - prev_point.x;
			CGFloat y = prevTouchPoint[i].y - prev_point.y;
			CGFloat l = x * x + y * y;
			if (l < length) {
				num = i;
				length = l;
			}
		}
		if (num < 10)
			flag[num] = YES;
	}
	UInt32 count = 0;
	for (UInt32 i=0; i<prevTouchPointCount; ++i) {
		if (flag[i])
			continue;
		prevTouchPoint[count] = prevTouchPoint[i];
		++count;
	}
	prevTouchPointCount = count;
	[self updateKeyDown];
	[self setNeedsDisplay];

	(void)[target performSelector:action withObject:self];
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
	prevTouchPointCount = 0;
	self.keyDown = 0;
	[self setNeedsDisplay];

	(void)[target performSelector:action withObject:self];
}

- (void)dealloc {
	[whiteKeyDown1 release];
	[whiteKeyDown2 release];
	[whiteKeyDown3 release];
	[whiteKeyDown4 release];
	[blackKeyDown release];
	[whiteKeyUp1 release];
	[whiteKeyUp2 release];
	[whiteKeyUp3 release];
	[whiteKeyUp4 release];
	[blackKeyUp release];
    [super dealloc];
}

- (void)setAction:(SEL)aSelector {
	action = aSelector;
}

- (void)setTarget:(id)anObject {
	target = anObject;
}

@end
